@@ -1,6 +1,7 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
 use glib::translate::*;
+use std::fmt;
 
 pub trait IsMiniObject:
     AsRef<Self::RefType> + FromGlibPtrFull<*mut Self::FfiType> + Send + Sync + 'static
@@ -553,22 +554,37 @@ impl MiniObject {
     }
 }
 
+impl fmt::Debug for MiniObject {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.as_ref().fmt(f)
+    }
+}
+
+impl fmt::Debug for MiniObjectRef {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("MiniObject")
+            .field("ptr", unsafe { &self.as_ptr() })
+            .field("type", &self.type_())
+            .finish()
+    }
+}
+
 impl MiniObjectRef {
     pub fn type_(&self) -> glib::Type {
         unsafe { from_glib((*self.as_ptr()).type_) }
     }
 
-    pub fn downcast_ref<T: IsMiniObject + glib::StaticType>(&self) -> Option<&T> {
+    pub fn downcast_ref<T: IsMiniObject + glib::StaticType>(&self) -> Option<&T::RefType> {
         if self.type_().is_a(T::static_type()) {
-            unsafe { Some(&*(self as *const Self as *const T)) }
+            unsafe { Some(&*(self as *const Self as *const T::RefType)) }
         } else {
             None
         }
     }
 
-    pub fn downcast_mut<T: IsMiniObject + glib::StaticType>(&mut self) -> Option<&mut T> {
+    pub fn downcast_mut<T: IsMiniObject + glib::StaticType>(&mut self) -> Option<&mut T::RefType> {
         if self.type_().is_a(T::static_type()) {
-            unsafe { Some(&mut *(self as *mut Self as *mut T)) }
+            unsafe { Some(&mut *(self as *mut Self as *mut T::RefType)) }
         } else {
             None
         }
