@@ -1,5 +1,7 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
+#![allow(clippy::upper_case_acronyms)]
+
 use glib::{Date, StaticType, ToValue};
 
 use num_rational::Rational32;
@@ -16,6 +18,7 @@ use once_cell::sync::Lazy;
 use crate::Buffer;
 use crate::DateTime;
 use crate::Sample;
+use crate::Structure;
 
 use crate::date_time_serde;
 use crate::value::*;
@@ -34,8 +37,9 @@ pub(crate) static INT_RANGE_I64_OTHER_TYPE_ID: Lazy<glib::Type> =
 pub(crate) static LIST_OTHER_TYPE_ID: Lazy<glib::Type> = Lazy::new(List::static_type);
 pub(crate) static SAMPLE_OTHER_TYPE_ID: Lazy<glib::Type> = Lazy::new(Sample::static_type);
 pub(crate) static BUFFER_OTHER_TYPE_ID: Lazy<glib::Type> = Lazy::new(Buffer::static_type);
+pub(crate) static STRUCTURE_OTHER_TYPE_ID: Lazy<glib::Type> = Lazy::new(Structure::static_type);
 
-impl<'a> Serialize for Fraction {
+impl Serialize for Fraction {
     fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
         self.0.serialize(serializer)
     }
@@ -83,6 +87,8 @@ macro_rules! ser_value (
                     ser_some_value!($value, Array, $ser_closure)
                 } else if *BITMASK_OTHER_TYPE_ID == type_id {
                     ser_some_value!($value, Bitmask, $ser_closure)
+                } else if *STRUCTURE_OTHER_TYPE_ID == type_id {
+                    ser_some_value!($value, Structure, $ser_closure)
                 } else if *DATE_OTHER_TYPE_ID == type_id {
                     ser_opt_value!($value, Date, |type_, value: Option<Date>| {
                         // Need to wrap the `glib::Date` in new type `date_time_serde::Date` first
@@ -148,7 +154,7 @@ impl Serialize for SendValue {
 
 macro_rules! impl_ser_send_value_collection (
     ($t:ident) => (
-        impl<'a> Serialize for $t<'a> {
+        impl Serialize for $t {
             fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
                 let send_value_vec = unsafe {
                     &*(self.as_slice() as *const [glib::SendValue] as *const [SendValue])
@@ -211,6 +217,7 @@ macro_rules! de_send_value(
             "String" => de_opt_send_value!($type_name, $seq, String),
             "Array" => de_some_send_value!($type_name, $seq, Array),
             "Bitmask" => de_some_send_value!($type_name, $seq, Bitmask),
+            "Structure" => de_some_send_value!($type_name, $seq, Structure),
             "Date" => {
                 // Need to deserialize as `date_time_serde::Date` new type
                 // See comment in `date_time_serde.rs`
@@ -268,11 +275,11 @@ impl<'de> Deserialize<'de> for SendValue {
 
 macro_rules! impl_de_send_value_collection (
     ($t:ident) => {
-        impl<'a, 'de> Deserialize<'de> for $t<'a> {
+        impl<'de> Deserialize<'de> for $t {
             fn deserialize<D: Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
                 skip_assert_initialized!();
                 let send_value_vec = Vec::<SendValue>::deserialize(deserializer)?;
-                Ok($t::from_owned(unsafe{
+                Ok($t::from_values(unsafe{
                     mem::transmute::<Vec<SendValue>, Vec<glib::SendValue>>(send_value_vec)
                 }))
             }
@@ -292,6 +299,7 @@ mod tests {
     use crate::FractionRange;
     use crate::IntRange;
     use crate::List;
+    use crate::Structure;
 
     use glib::{Date, DateMonth};
 
@@ -299,7 +307,7 @@ mod tests {
     fn test_serialize_simple() {
         crate::init().unwrap();
 
-        let pretty_config = ron::ser::PrettyConfig::new().with_new_line("".to_string());
+        let pretty_config = ron::ser::PrettyConfig::new().new_line("".to_string());
 
         // Fraction
         let fraction = Fraction::new(1, 3);
@@ -320,7 +328,7 @@ mod tests {
         assert_eq!(r#"{"min":[1,3],"max":[1,2]}"#.to_owned(), res);
 
         // IntRange
-        let int_range = IntRange::<i32>::with_step(0, 42, 21);
+        let int_range = IntRange::with_step(0, 42, 21);
         let res = ron::ser::to_string_pretty(&int_range, pretty_config.clone());
         assert_eq!(Ok("(    min: 0,    max: 42,    step: 21,)".to_owned()), res,);
 
@@ -330,35 +338,43 @@ mod tests {
         // Bitmask
         let bitmask = Bitmask::new(1024 + 128 + 32);
 
-        let res = ron::ser::to_string_pretty(&bitmask, pretty_config);
+        let res = ron::ser::to_string_pretty(&bitmask, pretty_config.clone());
         assert_eq!(Ok("(1184)".to_owned()), res);
 
         let res = serde_json::to_string(&bitmask).unwrap();
         assert_eq!("1184".to_owned(), res);
+
+        // Nested structure
+        let s = Structure::builder("foobar")
+            .field("foo", 42)
+            .field("baz", Structure::builder("baz").field("foo", 43).build())
+            .build();
+
+        let res = ron::ser::to_string_pretty(&s, pretty_config);
+        assert_eq!(
+            Ok(r#"("foobar", [    ("foo", "i32", 42),    ("baz", "Structure", ("baz", [        ("foo", "i32", 43),    ])),])"#.to_owned()),
+            res
+        );
+
+        let res = serde_json::to_string(&s).unwrap();
+        assert_eq!(
+            r#"["foobar",[["foo","i32",42],["baz","Structure",["baz",[["foo","i32",43]]]]]]"#
+                .to_owned(),
+            res
+        );
     }
 
     #[test]
     fn test_serialize_collections() {
         crate::init().unwrap();
 
-        let pretty_config = ron::ser::PrettyConfig::new().with_new_line("".to_string());
+        let pretty_config = ron::ser::PrettyConfig::new().new_line("".to_string());
 
         // Array
         let value_13 = Fraction::new(1, 3);
         let value_12 = Fraction::new(1, 2);
-        let value_str = "test str";
-        let value_str_none: Option<&str> = None;
-        let value_date = Date::new_dmy(19, DateMonth::August, 2019).unwrap();
-        let value_date_none: Option<Date> = None;
 
-        let array = Array::new(&[
-            &value_13,
-            &value_12,
-            &value_str,
-            &value_str_none,
-            &value_date,
-            &value_date_none,
-        ]);
+        let array = Array::new(&[&value_13, &value_12]);
 
         let res = ron::ser::to_string_pretty(&array, pretty_config.clone());
         assert_eq!(
@@ -366,10 +382,6 @@ mod tests {
                 r#"["#,
                 r#"    ("Fraction", (1, 3)),"#,
                 r#"    ("Fraction", (1, 2)),"#,
-                r#"    ("String", Some("test str")),"#,
-                r#"    ("String", None),"#,
-                r#"    ("Date", Some(YMD(2019, 8, 19))),"#,
-                r#"    ("Date", None),"#,
                 r#"]"#
             )
             .to_owned()),
@@ -377,39 +389,16 @@ mod tests {
         );
 
         let res = serde_json::to_string(&array).unwrap();
-        assert_eq!(
-            r#"[["Fraction",[1,3]],["Fraction",[1,2]],["String","test str"],["String",null],["Date",{"YMD":[2019,8,19]}],["Date",null]]"#
-                .to_owned(),
-            res
-        );
+        assert_eq!(r#"[["Fraction",[1,3]],["Fraction",[1,2]]]"#.to_owned(), res);
 
         // List
         let value_12 = Fraction::new(1, 2);
-        let value_str = "test str";
-        let value_str_none: Option<&str> = None;
-        let value_date_time = DateTime::new(2f32, 2019, 8, 19, 13, 34, 42f64).unwrap();
-        let value_date_time_none: Option<DateTime> = None;
 
-        let list = List::new(&[
-            &value_12,
-            &value_str,
-            &value_str_none,
-            &value_date_time,
-            &value_date_time_none,
-        ]);
+        let list = List::new(&[&value_12]);
 
         let res = ron::ser::to_string_pretty(&list, pretty_config);
         assert_eq!(
-            Ok(concat!(
-                r#"["#,
-                r#"    ("Fraction", (1, 2)),"#,
-                r#"    ("String", Some("test str")),"#,
-                r#"    ("String", None),"#,
-                r#"    ("DateTime", Some(YMDhmsTz(2019, 8, 19, 13, 34, 42, 2))),"#,
-                r#"    ("DateTime", None),"#,
-                r#"]"#
-            )
-            .to_owned()),
+            Ok(concat!(r#"["#, r#"    ("Fraction", (1, 2)),"#, r#"]"#).to_owned()),
             res,
         );
     }
@@ -463,6 +452,21 @@ mod tests {
         let bitmask_json = "1184";
         let bitmask: Bitmask = serde_json::from_str(bitmask_json).unwrap();
         assert_eq!(bitmask_ref, bitmask);
+
+        // Nested structure
+        let s_ref = Structure::builder("foobar")
+            .field("foo", 42)
+            .field("baz", Structure::builder("baz").field("foo", 43).build())
+            .build();
+
+        let s_ron = r#"("foobar", [    ("foo", "i32", 42),    ("baz", "Structure", ("baz", [        ("foo", "i32", 43),    ])),])"#;
+        let s: Structure = ron::de::from_str(s_ron).unwrap();
+        assert_eq!(s_ref, s);
+
+        let s_json =
+            r#"["foobar",[["foo","i32",42],["baz","Structure",["baz",[["foo","i32",43]]]]]]"#;
+        let s: Structure = serde_json::from_str(s_json).unwrap();
+        assert_eq!(s_ref, s);
     }
 
     #[test]
@@ -491,7 +495,7 @@ mod tests {
         );
 
         // IntRange
-        let int_range = IntRange::<i32>::with_step(0, 42, 21);
+        let int_range = IntRange::with_step(0, 42, 21);
         let int_range_ser = ron::ser::to_string(&int_range).unwrap();
         let int_range_de: IntRange<i32> = ron::de::from_str(int_range_ser.as_str()).unwrap();
         assert_eq!(int_range_de.min(), int_range.min());
@@ -503,6 +507,15 @@ mod tests {
         let bitmask_ser = ron::ser::to_string(&bitmask).unwrap();
         let bitmask_de: Bitmask = ron::de::from_str(bitmask_ser.as_str()).unwrap();
         assert_eq!(bitmask_de, bitmask);
+
+        // Nested structure
+        let s = Structure::builder("foobar")
+            .field("foo", 42)
+            .field("baz", Structure::builder("baz").field("foo", 43).build())
+            .build();
+        let s_ser = ron::ser::to_string(&s).unwrap();
+        let s_de: Structure = ron::de::from_str(s_ser.as_str()).unwrap();
+        assert_eq!(s_de, s);
     }
 
     #[allow(clippy::cognitive_complexity)]
@@ -542,7 +555,7 @@ mod tests {
             .is_none());
 
         assert_eq!(
-            Date::new_dmy(19, DateMonth::August, 2019).unwrap(),
+            Date::from_dmy(19, DateMonth::August, 2019).unwrap(),
             slice[4].get::<Date>().expect("slice[4]")
         );
 
@@ -572,7 +585,7 @@ mod tests {
             .is_none());
 
         assert_eq!(
-            Date::new_dmy(19, DateMonth::August, 2019).unwrap(),
+            Date::from_dmy(19, DateMonth::August, 2019).unwrap(),
             slice[4].get::<Date>().expect("slice[4]")
         );
 
@@ -622,19 +635,8 @@ mod tests {
         // Array
         let value_13 = Fraction::new(1, 3);
         let value_12 = Fraction::new(1, 2);
-        let value_str = "test str";
-        let value_str_none: Option<&str> = None;
-        let value_date = Date::new_dmy(19, DateMonth::August, 2019).unwrap();
-        let value_date_none: Option<Date> = None;
 
-        let array = Array::new(&[
-            &value_13,
-            &value_12,
-            &value_str,
-            &value_str_none,
-            &value_date,
-            &value_date_none,
-        ]);
+        let array = Array::new(&[&value_13, &value_12]);
         let array_ser = ron::ser::to_string(&array).unwrap();
 
         let array_de: Array = ron::de::from_str(array_ser.as_str()).unwrap();
@@ -652,37 +654,10 @@ mod tests {
         assert_eq!(fraction_de.0.numer(), fraction.0.numer());
         assert_eq!(fraction.0.denom(), fraction.0.denom());
 
-        assert_eq!(
-            slice_de[2].get::<String>().expect("slice_de[2]"),
-            slice[2].get::<String>().expect("slice[2]")
-        );
-
-        assert!(slice[3]
-            .get::<Option<String>>()
-            .expect("slice[3]")
-            .is_none());
-
-        assert_eq!(
-            slice_de[4].get::<Date>().expect("slice_de[4]"),
-            slice[4].get::<Date>().expect("slice[4]")
-        );
-
-        assert!(slice[5].get::<Option<Date>>().expect("slice[5]").is_none());
-
         // List
         let value_12 = Fraction::new(1, 2);
-        let value_str = "test str";
-        let value_str_none: Option<&str> = None;
-        let value_date_time = DateTime::new(2f32, 2019, 8, 19, 13, 34, 42f64).unwrap();
-        let value_date_time_none: Option<DateTime> = None;
 
-        let list = List::new(&[
-            &value_12,
-            &value_str,
-            &value_str_none,
-            &value_date_time,
-            &value_date_time_none,
-        ]);
+        let list = List::new(&[&value_12]);
         let list_ser = ron::ser::to_string(&list).unwrap();
 
         let list_de: List = ron::de::from_str(list_ser.as_str()).unwrap();
@@ -694,25 +669,5 @@ mod tests {
         let fraction = slice[0].get::<Fraction>().expect("slice[0]");
         assert_eq!(fraction_de.0.numer(), fraction.0.numer());
         assert_eq!(fraction_de.0.denom(), fraction.0.denom());
-
-        assert_eq!(
-            slice_de[1].get::<String>().expect("slice_de[1]"),
-            slice[1].get::<String>().expect("slice[1]")
-        );
-
-        assert!(slice[2]
-            .get::<Option<String>>()
-            .expect("slice[2]")
-            .is_none());
-
-        assert_eq!(
-            slice_de[3].get::<DateTime>().expect("slice_de[3]"),
-            slice[3].get::<DateTime>().expect("slice[3]")
-        );
-
-        assert!(slice[4]
-            .get::<Option<DateTime>>()
-            .expect("slice[4]")
-            .is_none());
     }
 }
